// Define gulp tasks.
module.exports = function (gulp, plugins, options) {

  'use strict';

  // var intercept = require('gulp-intercept');

  // Processor for linting is assigned to options so it can be reused later.
  options.processors = [
    // Options are defined in .stylelintrc.yaml file.
    plugins.stylelint(options.stylelint),
    plugins.reporter(options.processors.reporter)
  ];

  // Post CSS options.
  options.postcssOptions = [
    plugins.autoprefixer(options.autoprefixer),
    plugins.mqpacker({sort: true})
  ];

  // Gulp error handler.
  if (options.onError) {
    options.onErrorCallback = function (err) {
      plugins.notify.onError({
        title: options.onError.title + err.plugin,
        message: options.onError.message,
        sound: options.onError.sound
      })(err);
      this.emit('end');
    };
  }

  // Defining gulp tasks.

  // @todo - consider should we implement this.
  // @todo - For some other stuff check https://www.drupal.org/docs/8/theming/creating-automation-tools-for-custom-themes-gulpjs.
  // gulp.task('imagemin', function () {
  //   return gulp.src('./src/images/*')
  //     .pipe(imagemin({
  //       progressive: true,
  //       svgoPlugins: [{removeViewBox: false}],
  //       use: [pngquant()]
  //     }))
  //     .pipe(gulp.dest('./images'));
  // });

  gulp.task('sass', function () {
    var task = gulp.src(options.sass.src + '/**/*.scss');
    if (options.onError) {
      // Error management in gulp.
      task.pipe(plugins.plumber({errorHandler: options.onErrorCallback}));
    }
    task.pipe(plugins.sassInject(options.sass.injectVariables))
      // For testing of single file injection - check gulp-sass-inject/index.js
      // for more comments on this.
      // .pipe(intercept(function(file){
      //   console.log('FILE: ' + file.path );
      //   console.log('CONTENT: ' + file.contents.toString());
      //   return file;
      // }))
      .pipe(plugins.sassGlob())
      .pipe(plugins.sass({
        outputStyle: 'expanded',
        includePaths: options.sass.includePaths
      }))
      .pipe(plugins.postcss(options.postcssOptions))
      .pipe(plugins.stripCssComments())
      .pipe(gulp.dest(options.sass.dest));
    return task;
  });

  gulp.task('sass:dev', function () {
    var task = gulp.src(options.sass.src + '/**/*.scss', {sourcemaps: true});
    if (options.onError) {
      // Error management in gulp.
      task.pipe(plugins.plumber({errorHandler: options.onErrorCallback}));
    }
    task.pipe(plugins.sassInject(options.sass.injectVariables))
      .pipe(plugins.sassGlob())
      .pipe(plugins.sass({
        outputStyle: 'expanded',
        includePaths: options.sass.includePaths,
        sourceComments: 'map', sourceMap: 'sass'
      }))
      .pipe(plugins.postcss(options.postcssOptions))
      .pipe(gulp.dest(options.sass.dest, {sourcemaps: '.'}));
    return task;
  });

  gulp.task('sass:lint', function () {
    return gulp.src(options.sass.src + '/**/*.scss')
      .pipe(plugins.postcss(options.processors, {syntax: plugins.syntax_scss}));
  });

  gulp.task('clean:css', function () {
    return plugins.del([
      'css/**/*'
    ]);
  });

  // Gulp run all tasks.

  gulp.task('default', gulp.series('clean:css', 'sass:lint', 'sass'));
  gulp.task('prod', gulp.series('default'));
  gulp.task('dev', gulp.series('sass:dev'));
  gulp.task('clean', gulp.series('clean:css'));

  // Gulp watches.

  gulp.task('watch', function () {
    gulp.watch(options.sass.src + '/**/*.scss', ['sass']);
  });
  gulp.task('watch:dev', function () {
    gulp.watch(options.sass.src + '/**/*.scss', ['sass:dev']);
  });

};
